//
//  BMKSignInController.m
//  BMKSignInOCDemo
//
//  Created by v_chiyuanwei on 2020/7/16.
//  Copyright © 2020 baidu. All rights reserved.
//

#import "BMKSignInController.h"

/// 地图
#import <BaiduMapAPI_Map/BMKMapComponent.h>
/// 地理编码
#import <BaiduMapAPI_Search/BMKSearchComponent.h>
/// 工具
#import <BaiduMapAPI_Utils/BMKGeometry.h>
/// 定位
#import <BMKLocationkit/BMKLocationComponent.h>

#import "BMKSignInTabView.h"
#import "BMKLocInfoView.h"

@interface BMKSignInController () <BMKMapViewDelegate, BMKLocationManagerDelegate, BMKGeoCodeSearchDelegate> {
    CLLocationCoordinate2D _signCircleCenter; // 签到范围中心点坐标，假设为(40.055854, 116.306761)
    CGFloat _signRadius; // 签到范围半径，默认500米
}
/// 地图
@property (nonatomic, strong) BMKMapView *mapView;
/// 定位管理类
@property (nonatomic, strong) BMKLocationManager *locationManager;
/// 当前位置Annotation
@property (nonatomic, strong) BMKPointAnnotation *userLocAnnotation;
/// 签到位置Annotation
@property (nonatomic, strong) BMKPointAnnotation *signInLocAnnotation;
/// 签到范围。_signCircleCenter为中心，半径_signRadius的圆形overlay。
@property (nonatomic, strong) BMKCircle *circleOverlay;
/// 签到按钮
@property (nonatomic, strong) BMKSignInTabView *tabView;
/// 位置信息
@property (nonatomic, strong) BMKLocInfoView *locInfoView;
/// HUD
@property (nonatomic, strong) UILabel *alertLabel;
/// 当前位置坐标
@property (nonatomic, assign) CLLocationCoordinate2D userLocCoord;
/// 签到坐标
@property (nonatomic, assign) CLLocationCoordinate2D signInCoord;
/// 是否已经签到
@property (nonatomic, assign, getter=isSignIn) BOOL signIn;
/// 定位结果
@property (nonatomic, assign, getter=isLocSuccess) BOOL locSuccess;
/// 是否在签到范围内
@property (nonatomic, assign, getter=isInSignCircle) BOOL inSignCircle;

@end

@implementation BMKSignInController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"签到场景";
    
    [self initialize];
    [self setupViews];
    // 添加签到范围overlay
    [self addSignInOverlay];
    [self startLocation];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // 当mapView即将被显示的时候调用，恢复之前存储的mapView状态
    [self.mapView viewWillAppear];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // 当mapView即将被隐藏的时候调用，存储当前mapView的状态
    [self.mapView viewWillDisappear];
}

#pragma mark -
- (void)initialize {
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationController.navigationBar.translucent = NO;
    self.navigationController.navigationBar.barStyle = UIBarStyleBlack;
    self.navigationController.navigationBar.barTintColor = [UIColor colorWithRed:34.f / 255.f green:37.f / 255.f blue:61.f / 255.f alpha:1.f];
    [self.navigationController.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor whiteColor]}];
    
    _signCircleCenter = CLLocationCoordinate2DMake(40.055854, 116.306761);
    _signRadius = 500;
}

- (void)setupViews {
    [self.view addSubview:self.mapView];
    [self.view addSubview:self.tabView];
    [self.view addSubview:self.locInfoView];
    [self.view addSubview:self.alertLabel];
}

/// 签到范围，圆形Overlay
- (void)addSignInOverlay {
    [self.mapView addOverlay:self.circleOverlay];
}

#pragma mark -
/// 开始定位
- (void)startLocation {
    [self.locationManager requestLocationWithReGeocode:YES withNetworkState:YES completionBlock:^(BMKLocation * _Nullable location, BMKLocationNetworkState state, NSError * _Nullable error) {
        // 定位错误信息
        if (error) {
            self.locSuccess = NO;
            [self showMsg:@"获取当前位置失败"];
            NSLog(@"locError:{%zd - %@};", error.code, error.localizedDescription);
            return;
        }
        
        // 定位结果
        if (location) {
            CLLocationCoordinate2D coord = location.location.coordinate;
            NSLog(@"%f==%f", coord.latitude, coord.longitude);
            // 逆地理编码，获取当前位置周边POI
            [self rgcSearch:coord];
            // 设置地图中心点
            [self.mapView setCenterCoordinate:coord];
            // 更新当前位置数据
            [self updateUserLocation:coord];
            // 判断是否在签到范围
            self.inSignCircle = BMKCircleContainsCoordinate(coord, self->_signCircleCenter, self->_signRadius);
            
            if (!self.isInSignCircle) {
                [self showMsg:@"不在签到范围内"];
            }
            
            self.userLocCoord = coord;
            self.signInCoord = coord;
            self.locSuccess = YES;
        }
    }];
}

/// 更新当前位置数据
- (void)updateUserLocation:(CLLocationCoordinate2D)coord {
    // 经纬度转屏幕坐标
    CGPoint userLocScreenPoint = [self.mapView convertCoordinate:coord toPointToView:self.mapView];
    // 设置当前位置锁定到屏幕的坐标
    self.userLocAnnotation.screenPointToLock = userLocScreenPoint;

    if (![self.mapView.annotations containsObject:self.userLocAnnotation]) {
        [self.mapView addAnnotation:self.userLocAnnotation];
    }
}

/// 逆地理编码
- (void)rgcSearch:(CLLocationCoordinate2D)coord {
    // 初始化逆地理编码检索对象
    BMKGeoCodeSearch *search = [[BMKGeoCodeSearch alloc] init];
    // 设置正地理编码检索代理
    search.delegate = self;
    
    // 构造逆地理编码检索参数
    BMKReverseGeoCodeSearchOption *searchOption = [[BMKReverseGeoCodeSearchOption alloc] init];
    // 待解析的经纬度坐标（必选）
    searchOption.location = coord;
    // 设置POI召回半径,此处等于签到范围半径
    searchOption.radius = 500;
    // 是否访问最新版行政区划数据（仅对中国数据生效）
    searchOption.isLatestAdmin = YES;
    
    // 发起逆地理编码检索请求
    [search reverseGeoCode:searchOption];
}

/// 签到
- (void)signIn {
    if (!self.isInSignCircle) {
        [self showMsg:@"不在签到范围内"];
        return;
    }
    
    if (!self.isSignIn) {
        // 添加签到位置Annotation
        self.signInLocAnnotation.coordinate = self.signInCoord;
        [self.mapView addAnnotation:self.signInLocAnnotation];
        self.signIn = YES;
        [self showMsg:@"签到成功"];
    } else {
        [self showMsg:@"今日已签到"];
    }
}

/// alert msg
- (void)showMsg:(NSString *)msg {
    if (msg.length == 0 || !self.alertLabel.isHidden) return;
    
    CGSize msgSize = [msg boundingRectWithSize:CGSizeMake(self.view.frame.size.width - 40, 40) options:NSStringDrawingUsesFontLeading attributes:@{NSFontAttributeName: [UIFont systemFontOfSize:14]} context:nil].size;
    self.alertLabel.bounds = CGRectMake(0, 0, msgSize.width + 20, msgSize.height + 16);
    self.alertLabel.center = self.view.center;
    self.alertLabel.text = msg;
    self.alertLabel.hidden = NO;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        self.alertLabel.hidden = YES;
    });
}

#pragma mark - <BMKMapViewDelegate>
/// 根据anntation生成对应的View
- (BMKAnnotationView *)mapView:(BMKMapView *)mapView viewForAnnotation:(id<BMKAnnotation>)annotation {
    
    if (annotation == self.userLocAnnotation) { // 当前位置
        BMKAnnotationView *userLocAnnotationView = [mapView dequeueReusableAnnotationViewWithIdentifier:@"userLocAnnotationView"];
        if (!userLocAnnotationView) {
            userLocAnnotationView = [[BMKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"userLocAnnotationView"];
            
        }
        userLocAnnotationView.image = [UIImage imageNamed:@"red_annotation_icon"];
        return userLocAnnotationView;
    } else if (annotation == self.signInLocAnnotation) { // 签到位置
        BMKAnnotationView *signInLocAnnotationView = [mapView dequeueReusableAnnotationViewWithIdentifier:@"signInLocAnnotation"];
        if (!signInLocAnnotationView) {
            signInLocAnnotationView = [[BMKAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"signInLocAnnotation"];
        }

        signInLocAnnotationView.image = [UIImage imageNamed:@"blue_annotation_icon"];
        signInLocAnnotationView.displayPriority = BMKFeatureDisplayPriorityDefaultHigh;
        return signInLocAnnotationView;
    }
    
    return nil;
}

/// 根据overlay生成对应的View
- (BMKOverlayView *)mapView:(BMKMapView *)mapView viewForOverlay:(id<BMKOverlay>)overlay {
    
    if (overlay == self.circleOverlay) {
        BMKCircleView* circleView = [[BMKCircleView alloc] initWithOverlay:overlay];
        circleView.fillColor = [[UIColor blueColor] colorWithAlphaComponent:0.2f];
        circleView.strokeColor = [[UIColor blueColor] colorWithAlphaComponent:0.7f];
        circleView.lineWidth = 1.5f;
        
        return circleView;
    }
    
    return nil;
}

/// 地图区域改变完成后会调用此方法
- (void)mapView:(BMKMapView *)mapView regionDidChangeAnimated:(BOOL)animated reason:(BMKRegionChangeReason)reason {
    if (!self.isInSignCircle) return;
    if (!self.isLocSuccess) return;
    
    // 将当前位置屏幕坐标转换为经纬度坐标
    CLLocationCoordinate2D userLocCoord = [self.mapView convertPoint:self.userLocAnnotation.screenPointToLock toCoordinateFromView:self.mapView];
    // 判断当前位置是否在签到范围内
    BOOL isContains = BMKCircleContainsCoordinate(userLocCoord, self.circleOverlay.coordinate, self.circleOverlay.radius);
    if (isContains) { // 在签到范围内
        self.signInCoord = userLocCoord;
    } else { // 在签到范围外
        [self showMsg:@"微调距离不可超过500米"];
        [mapView setCenterCoordinate:self.userLocCoord animated:YES];
    }
}

#pragma mark - <BMKGeoCodeSearchDelegate>
/// 返回逆地理编码搜索结果
- (void)onGetReverseGeoCodeResult:(BMKGeoCodeSearch *)searcher result:(BMKReverseGeoCodeSearchResult *)result errorCode:(BMKSearchErrorCode)error {
    
    if (error == BMK_SEARCH_NO_ERROR) { // 检索成功
        // 展示当前位置周边POI
        self.locInfoView.dataArr = result.poiList;
    }
}

#pragma mark - getter
/// 地图
- (BMKMapView *)mapView {
    if (!_mapView) {
        _mapView = [[BMKMapView alloc] initWithFrame: CGRectMake(0, 0, SCREEN_WIDTH, CONTENT_HEIGHT - 200)];
        // 地图比例尺级别
        _mapView.zoomLevel = 17;
        // 设置代理
        _mapView.delegate = self;
    }
    return _mapView;
}

/// 定位管理类
- (BMKLocationManager *)locationManager {
    if (!_locationManager) {
        _locationManager = [[BMKLocationManager alloc] init];
        // 设置返回位置的坐标系类型
        _locationManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        // 设置距离过滤参数
        _locationManager.distanceFilter = 5.0;
        // 设置预期精度参数
        _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        // 设置应用位置类型
        _locationManager.activityType = CLActivityTypeAutomotiveNavigation;
        // 设置是否自动停止位置更新
        _locationManager.pausesLocationUpdatesAutomatically = NO;
        // 设置位置获取超时时间
        _locationManager.locationTimeout = 10;
        // 设置获取地址信息超时时间
        _locationManager.reGeocodeTimeout = 10;
        // 设置delegate
        _locationManager.delegate = self;
    }
    return _locationManager;
}

/// 当前位置标注
- (BMKPointAnnotation *)userLocAnnotation {
    if (!_userLocAnnotation) {
        _userLocAnnotation = [[BMKPointAnnotation alloc] init];
        _userLocAnnotation.isLockedToScreen = YES;
    }
    return _userLocAnnotation;
}

/// 签到位置标注
- (BMKPointAnnotation *)signInLocAnnotation {
    if (!_signInLocAnnotation) {
        _signInLocAnnotation = [[BMKPointAnnotation alloc] init];
    }
    return _signInLocAnnotation;
}

/// 签到范围
- (BMKCircle *)circleOverlay {
    if (!_circleOverlay) {
        _circleOverlay = [[BMKCircle alloc] init];
        _circleOverlay.coordinate = _signCircleCenter;
        _circleOverlay.radius = _signRadius;
    }
    return _circleOverlay;
}

/// 定位/签到 按钮
- (BMKSignInTabView *)tabView {
    if (!_tabView) {
        _tabView = [[BMKSignInTabView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, 60)];
        __weak typeof(self) weakSelf = self;
        _tabView.didClickTabBlock = ^(NSInteger idx) {
            if (idx == 0) { // 获取位置
                [weakSelf startLocation];
            } else { // 签到
                [weakSelf signIn];
            }
        };
    }
    return _tabView;
}

/// 定位结果
- (BMKLocInfoView *)locInfoView {
    if (!_locInfoView) {
        _locInfoView = [[BMKLocInfoView alloc] initWithFrame:CGRectMake(0, CONTENT_HEIGHT - 200, SCREEN_WIDTH, 200)];
        __weak typeof(self) weakSelf = self;
        _locInfoView.didSelectRowBlock = ^(BMKPoiInfo *poiInfo) {
            // 记录签到坐标
            weakSelf.signInCoord = poiInfo.pt;
            // 设置地图中心
            weakSelf.mapView.centerCoordinate = poiInfo.pt;
        };
    }
    return _locInfoView;
}

/// HUD
- (UILabel *)alertLabel {
    if (!_alertLabel) {
        _alertLabel = [[UILabel alloc] init];
        _alertLabel.backgroundColor = [UIColor colorWithWhite:0.0f alpha:0.5];
        _alertLabel.font = [UIFont systemFontOfSize:14];
        _alertLabel.textColor = [UIColor whiteColor];
        _alertLabel.textAlignment = NSTextAlignmentCenter;
        _alertLabel.layer.cornerRadius = 8;
        _alertLabel.layer.masksToBounds = YES;
        _alertLabel.hidden = YES;
    }
    return _alertLabel;
}

@end

